* {{ 
  font-family: 'Lato' !important; 
}}

/*
QPushButton {{
  color: {QTMATERIAL_SECONDARYCOLOR};
  text-transform: none;
  background-color: {QTMATERIAL_PRIMARYCOLOR};
}}
*/

QLineEdit {{
  color: #ffffff; 
  border: 1px solid #5c5c5c;
}}
QTextEdit {{
  color: #ffffff; 
  border: 1px solid #5c5c5c;
}}

QCheckBox::indicator:checked {{
  background-color: {QTMATERIAL_PRIMARYCOLOR}; 
}}
QCheckBox::indicator:unchecked {{ 
  background-color: {QTMATERIAL_SECONDARYCOLOR}; 
}}

QRadioButton::indicator:checked {{
  background-color: {QTMATERIAL_PRIMARYCOLOR}; 
}} 
QRadioButton::indicator:unchecked {{
  background-color: {QTMATERIAL_SECONDARYCOLOR}; 
}}

QMenu::indicator:checked {{ 
  background-color: {QTMATERIAL_PRIMARYCOLOR}; 
}} 
QMenu::indicator:unchecked {{ 
  background-color: {QTMATERIAL_SECONDARYCOLOR}; 
}}